﻿using System;
using System.Text;
using System.Windows.Forms;
using SDRSharp.Common;
using SDRSharp.Radio;

namespace SDRSharp.DDETracker
{
    public partial class DdeTrackingPanel : UserControl
    {
        private readonly ISharpControl _control;
        private readonly DdeTrackingConfigForm _configForm;
        private WispDdeClient _ddeClient;
        private bool _isConnected;        

        public bool IsConnected
        {
            get { return _isConnected; }
        }

        public DdeTrackingPanel(ISharpControl control)
        {
            InitializeComponent();
            _control = control;
            _configForm = new DdeTrackingConfigForm(this);
        }

        private void TrackerAdviseArrived(object sender, AdviseArrivedEventArgs args)
        {
            Invoke(new MethodInvoker(() =>
            {
                if (_control.IsPlaying)
                {
                    if (args.DownLinkFrequency != _control.Frequency && args.DownLinkFrequency != 0)
                    {
                        _control.Frequency = args.DownLinkFrequency;
                    }
                    if (args.ModeValid && args.Mode != _control.DetectorType)
                    {
                        _control.DetectorType = args.Mode;
                    }
                }

                nameValueLbl.Text = string.Format("{0}", args.Name);
                eleValueLbl.Text = string.Format("{0}°", args.Elevation);
                aziValueLbl.Text = string.Format("{0}°", args.Azimuth);
                frequencyValueLbl.Text = string.Format("{0} Hz", args.DownLinkFrequency);
            }));
        }

        void TrackerDisconnectHandler(object sender)
        {
            Invoke(new MethodInvoker(() => { Disconnect(); }));
        }

        #region GUI Control Handlers

        private void optionsBtn_Click(object sender, EventArgs e)
        {
            _configForm.Show();
        }

        private void connectBtn_Click(object sender, EventArgs e)
        {
            if (_ddeClient == null)
            {
                _ddeClient = new WispDdeClient(_configForm.ApplicationName, _configForm.LinkTopic, _configForm.LinkItem);
                try
                {
                    _ddeClient.Connect();
                    _ddeClient.AdviseArrivedEvent += TrackerAdviseArrived;
                    _ddeClient.ServerDisconnectEvent += TrackerDisconnectHandler;
                    _isConnected = true;
                    connectBtn.Text = "Disconnect";
                    optionBtn.Enabled = false;
                }
                catch
                {
                    MessageBox.Show("Unable to connect to DDE source", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _ddeClient = null;
                    _isConnected = false;
                    optionBtn.Enabled = true;
                }                
            }
            else
            {
                Disconnect();
            }
            
        }
        
        #endregion
     
        private void Disconnect()
        {
            if (_ddeClient != null)
            {
                _ddeClient.Disconnect();
                _ddeClient = null;
            }
            _isConnected = false;            
            connectBtn.Text = "Connect";
            optionBtn.Enabled = true;
            eleValueLbl.Text = "0°";
            aziValueLbl.Text = "0°";
            nameValueLbl.Text = "None";
            frequencyValueLbl.Text = "0 Hz";
        }
    }
}
